import c4d
from c4d import gui
#Welcome to the world of Python

def iter_tree(obj, include_first=True):
    if include_first:
        yield obj

    for child in obj.GetChildren():
        for o in iter_tree(child, True):
            yield o

def iter_all(doc):
    for obj in doc.GetObjects():
        for o in iter_tree(obj):
            yield o

def main():
    doc = c4d.documents.GetActiveDocument()
        
    c4d.StatusSetText("Evaluating Render Geometry...")
    
    # evaluate the document with the external render flag set, so all rendering
    # geometry will be generated
    doc.ExecutePasses(None, True, True, True, c4d.BUILDFLAGS_EXTERNALRENDERER)
                
    for obj in iter_all(doc):
        # Since we're making a few assumptions about the material assignements that might not hold
        # true for the general case, we're only operating on Laubwerk Plant objects.
        if obj.GetType() != 1028605:
            continue
        
        c4d.StatusSetText("Processing Object (" + obj.GetName() + ")")
        
        cacheObj = obj.GetCache()
        
        for tag in reversed(obj.GetTags()):
            if tag.GetType() == c4d.Ttexture:
                restriction = tag.GetDataInstance().GetString(c4d.TEXTURETAG_RESTRICTION)
                if len(restriction) == 0:
                    # actually there shouldn't be texture tags without restriction
                    continue;
                
                # check if there is a matching selection
                foundSelection = False
                for cacheTag in cacheObj.GetTags():
                    if cacheTag.GetName() == restriction:
                        print cacheTag.GetName()
                        foundSelection = True
                        break
                
                # materials that we didn't find a proper selection for, are moved to the beginning
                if not foundSelection:
                    tag.Remove()
                    obj.InsertTag(tag)

    c4d.StatusSetText("Evaluating Editor Geometry...")

    doc.ExecutePasses(None, True, True, True, c4d.BUILDFLAGS_0)
    c4d.EventAdd()
    
    c4d.StatusClear()

if __name__=='__main__':
    main()
